# lingo - payment and billing system
# Copyright (C) 2024  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.urls import reverse
from django.views.generic import CreateView, DeleteView, TemplateView, UpdateView

from lingo.invoicing.forms import PaymentCancellationReasonForm
from lingo.invoicing.models import PaymentCancellationReason


class ReasonListView(TemplateView):
    template_name = 'lingo/invoicing/manager_cancellation_reason_list.html'

    def get_context_data(self, **kwargs):
        kwargs.update(
            {
                'payment_reason_list': PaymentCancellationReason.objects.all().order_by('disabled', 'label'),
            }
        )
        return super().get_context_data(**kwargs)


reason_list = ReasonListView.as_view()


class PaymentReasonAddView(CreateView):
    template_name = 'lingo/invoicing/manager_payment_cancellation_reason_form.html'
    model = PaymentCancellationReason
    fields = ['label']

    def get_success_url(self):
        return '%s#open:payment' % reverse('lingo-manager-invoicing-cancellation-reason-list')


payment_reason_add = PaymentReasonAddView.as_view()


class PaymentReasonEditView(UpdateView):
    template_name = 'lingo/invoicing/manager_payment_cancellation_reason_form.html'
    model = PaymentCancellationReason
    form_class = PaymentCancellationReasonForm

    def get_success_url(self):
        return '%s#open:payment' % reverse('lingo-manager-invoicing-cancellation-reason-list')


payment_reason_edit = PaymentReasonEditView.as_view()


class PaymentReasonDeleteView(DeleteView):
    template_name = 'lingo/manager_confirm_delete.html'
    model = PaymentCancellationReason

    def get_queryset(self):
        return PaymentCancellationReason.objects.filter(payment__isnull=True)

    def get_success_url(self):
        return '%s#open:payment' % reverse('lingo-manager-invoicing-cancellation-reason-list')


payment_reason_delete = PaymentReasonDeleteView.as_view()
